From 16a38306e053ed044aa6fafe73684fe9ec0f38c1 Mon Sep 17 00:00:00 2001
From: Wei xie <xiewei@linuxdeepin.com>
Date: Mon, 8 Jan 2018 11:47:20 +0800
Subject: [PATCH] wininet: Check null pointer in InternetGetSecurityInfoByURLW

Add InternetGetSecurityInfoByURLW tests

Signed-off-by: Wei xie <xiewei@linuxdeepin.com>
---
 dlls/wininet/internet.c   | 12 ++++++++++--
 dlls/wininet/tests/http.c | 16 ++++++++++++++++
 2 files changed, 26 insertions(+), 2 deletions(-)

diff --git a/dlls/wininet/internet.c b/dlls/wininet/internet.c
index 1f1bbbd6d4..a67b84ffa9 100644
--- a/dlls/wininet/internet.c
+++ b/dlls/wininet/internet.c
@@ -4462,6 +4462,11 @@ BOOL WINAPI InternetGetSecurityInfoByURLW(LPCWSTR lpszURL, PCCERT_CHAIN_CONTEXT
 
     TRACE("(%s %p %p)\n", debugstr_w(lpszURL), ppCertChain, pdwSecureFlags);
 
+    if (!ppCertChain && !pdwSecureFlags) {
+        SetLastError(ERROR_INVALID_PARAMETER);
+        return FALSE;
+    }
+
     url.dwHostNameLength = 1;
     res = InternetCrackUrlW(lpszURL, 0, 0, &url);
     if(!res || url.nScheme != INTERNET_SCHEME_HTTPS) {
@@ -4480,8 +4485,11 @@ BOOL WINAPI InternetGetSecurityInfoByURLW(LPCWSTR lpszURL, PCCERT_CHAIN_CONTEXT
 
         chain_dup = CertDuplicateCertificateChain(server->cert_chain);
         if(chain_dup) {
-            *ppCertChain = chain_dup;
-            *pdwSecureFlags = server->security_flags & _SECURITY_ERROR_FLAGS_MASK;
+            if (ppCertChain) *ppCertChain = chain_dup;
+            else CertFreeCertificateChain(chain_dup);
+
+            if (pdwSecureFlags)
+                *pdwSecureFlags = server->security_flags & _SECURITY_ERROR_FLAGS_MASK;
         }else {
             res = FALSE;
         }
diff --git a/dlls/wininet/tests/http.c b/dlls/wininet/tests/http.c
index ca1173e940..2b76511c77 100644
--- a/dlls/wininet/tests/http.c
+++ b/dlls/wininet/tests/http.c
@@ -5590,6 +5590,22 @@ static void _test_security_info(unsigned line, const char *urlc, DWORD error, DW
         ok_(__FILE__,line)(chain != NULL, "chain = NULL\n");
         ok_(__FILE__,line)(flags == ex_flags, "flags = %x\n", flags);
         CertFreeCertificateChain(chain);
+
+        SetLastError(0xdeadbeef);
+        res = pInternetGetSecurityInfoByURLA(url, NULL, NULL);
+        ok_(__FILE__,line)(!res && GetLastError() == ERROR_INVALID_PARAMETER,
+                           "InternetGetSecurityInfoByURLA returned: %x(%u), expected failed %u\n",
+                            res, GetLastError(), ERROR_INVALID_PARAMETER);
+
+        SetLastError(0);
+        res = pInternetGetSecurityInfoByURLA(url, &chain, NULL);
+        ok_(__FILE__,line)(res && GetLastError() == 0,
+                           "InternetGetSecurityInfoByURLA failed returned: %x(%u), expected success\n", res, GetLastError());
+
+        SetLastError(0);
+        res = pInternetGetSecurityInfoByURLA(url, NULL, &flags);
+        ok_(__FILE__,line)(res && GetLastError() == 0,
+                           "InternetGetSecurityInfoByURLA failed returned: %x(%u), expected success\n", res, GetLastError());
     }else {
         ok_(__FILE__,line)(!res && GetLastError() == error,
                            "InternetGetSecurityInfoByURLA returned: %x(%u), expected %u\n", res, GetLastError(), error);
-- 
2.11.0

